@@ -30,6 +30,9 @@ class StandardDataset(data.Dataset):
                 F.interpolate(torch.Tensor(self.data[np.newaxis, np.newaxis, ...]), size=tuple(target_size),
                               mode='area')[0, 0, ...].numpy()
 
+        # data is rescaled to [0,1]
+        self.data = self.data / 255
+
     def __getitem__(self, i):
         pass
 
@@ -139,6 +142,9 @@ class VolumeDataset(data.Dataset):
                 F.interpolate(torch.Tensor(self.data[np.newaxis, np.newaxis, ...]), size=tuple(target_size),
                               mode='area')[0, 0, ...].numpy()
 
+        # data is rescaled to [0,1]
+        self.data = self.data / 255
+
     def __getitem__(self, i):
         pass
 
@@ -256,6 +262,9 @@ class MultiVolumeDataset(data.Dataset):
                     F.interpolate(torch.Tensor(data[np.newaxis, np.newaxis, ...]), size=tuple(target_size),
                                   mode='area')[0, 0, ...].numpy()
 
+            # data is rescaled to [0,1]
+            data /= 255
+
             self.data.append(data)
             self.data_sizes.append(data.size)
         self.data_sizes = np.array(self.data_sizes)
